<?php
namespace FiloBlu\Checkproducts\Cron;
use \Psr\Log\LoggerInterface;

class Pricecheck {
    protected $_logger;
    protected $_productRepository;
    protected $_searchCriteriaBuilder;
    protected $_scopeConfig;
    protected $_storeRepository;
    protected $_objectManager;
    protected $_linkManagement;
    protected $_helper;

    public function __construct(
        LoggerInterface $logger,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreRepository $storeRepository,
        \Magento\Framework\ObjectManagerInterface $objectmanager,
        \Magento\ConfigurableProduct\Api\LinkManagementInterface $linkManagement,
        \FiloBlu\Checkproducts\Helper\Data $helper
    ) {
        $this->_logger = $logger;
        $this->_productRepository = $productRepository;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->_scopeConfig = $scopeConfig;
        $this->_storeRepository = $storeRepository;
        $this->_objectManager = $objectmanager;
        $this->_linkManagement = $linkManagement;
        $this->_helper = $helper;
    }

    /**
     * Write to system.log
     *
     * @return void
     */

    public function execute() {

        $active = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_general/checkproducts_enabled');
        if(!$active){
            return;
        }

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();

        $sql = "SELECT * FROM catalog_product_entity AS product WHERE product.".$this->_helper->getEntityField()." IN ( 
                    SELECT DISTINCT cpei.".$this->_helper->getEntityField()." FROM `catalog_product_entity_int` cpei
                    WHERE cpei.attribute_id = (
                          SELECT eav.attribute_id FROM `eav_attribute` AS eav 
                          WHERE eav.`attribute_code` LIKE 'status' AND eav.entity_type_id = 4
                    ) 
                    AND cpei.value = 1
                    AND cpei.".$this->_helper->getEntityField()." NOT IN (
                        SELECT pei.".$this->_helper->getEntityField()." FROM `catalog_product_entity_int` pei
                        WHERE pei.attribute_id = (
                              SELECT eav.attribute_id FROM `eav_attribute` AS eav 
                              WHERE eav.`attribute_code` LIKE 'manual_status' AND eav.entity_type_id = 4
                        ) 
                        AND pei.value = 1 AND pei.store_id = 0
                    )
                )
                AND product.type_id = 'configurable'";

        $results = $connection->fetchAll($sql);
        $configurableProducts = [];
        foreach ($results as $res){
            $configurableProducts[] = $res[$this->_helper->getEntityField()];
        }

        $sql = "SELECT * FROM catalog_product_entity AS product WHERE product.".$this->_helper->getEntityField()." IN ( 
                    SELECT DISTINCT cpei.".$this->_helper->getEntityField()." FROM `catalog_product_entity_int` cpei
                    WHERE cpei.attribute_id = (
                          SELECT eav.attribute_id FROM `eav_attribute` AS eav 
                          WHERE eav.`attribute_code` LIKE 'status' AND eav.entity_type_id = 4
                    ) 
                    AND cpei.value = 1
                    AND cpei.".$this->_helper->getEntityField()." NOT IN (
                        SELECT pei.".$this->_helper->getEntityField()." FROM `catalog_product_entity_int` pei
                        WHERE pei.attribute_id = (
                              SELECT eav.attribute_id FROM `eav_attribute` AS eav 
                              WHERE eav.`attribute_code` LIKE 'manual_status' AND eav.entity_type_id = 4
                        ) 
                        AND pei.value = 1 AND pei.store_id = 0
                    )
                )
                AND product.type_id = 'simple'";

        $results = $connection->fetchAll($sql);
        $simpleProducts = [];
        foreach ($results as $res){
            $simpleProducts[] = $res[$this->_helper->getEntityField()];
        }

        $priceCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_simple/checkproducts_simple_price', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $priceConfCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_configurable/checkproducts_configurable_price', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $imageSimpleCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_simple/checkproducts_simple_image', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $imageCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_configurable/checkproducts_configurable_image', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $categorySimpleCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_simple/checkproducts_simple_categories', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $categoryConfCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_configurable/checkproducts_configurable_categories', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $stores = $this->_storeRepository->getList();
        $problems = [];
        $toDisable = [];
        foreach ($stores as $store) {
            $check = [];
            if($priceCheck){

                $sqlPrice = "SELECT cpe.".$this->_helper->getEntityField().", cpe.sku
                             FROM catalog_product_entity as cpe
                             WHERE cpe.".$this->_helper->getEntityField()." IN 
                            (
                                SELECT deci.".$this->_helper->getEntityField()." 
                                FROM catalog_product_entity_decimal AS deci 
                                WHERE attribute_id = 
                                (
                                 SELECT eav.attribute_id FROM `eav_attribute` AS eav
                                 WHERE eav.`attribute_code` LIKE 'price' AND eav.entity_type_id = 4
                                 )
                                AND value = 0
                                AND store_id = " . $store["store_id"] . "
                                AND ".$this->_helper->getEntityField()." IN (" . implode(",", $simpleProducts) . ")
                            )";

                $resultsPrice = $connection->fetchAll($sqlPrice);
                foreach ($resultsPrice as $rp){
                    $problems[$rp['sku']]['price'][$store["code"]] = false;
                    $toDisable[$rp[$this->_helper->getEntityField()]] = $rp[$this->_helper->getEntityField()];
                }
            }

            /*$check = [];
            if($priceConfCheck){

                $sqlPrice = "SELECT cpe.".$this->_helper->getEntityField().", cpe.sku
                             FROM catalog_product_entity as cpe
                             WHERE cpe.".$this->_helper->getEntityField()." IN 
                            (
                                SELECT deci.".$this->_helper->getEntityField()." 
                                FROM catalog_product_entity_decimal AS deci 
                                WHERE attribute_id = 
                                (
                                 SELECT eav.attribute_id FROM `eav_attribute` AS eav
                                 WHERE eav.`attribute_code` LIKE 'price' AND eav.entity_type_id = 4
                                 )
                                AND value = 0
                                AND store_id = " . $store["store_id"] . "
                                AND ".$this->_helper->getEntityField()." IN (" . implode(",", $configurableProducts) . ")
                            )";

                $resultsPrice = $connection->fetchAll($sqlPrice);
                foreach ($resultsPrice as $rp){
                    $problems[$rp['sku']]['price'][$store["code"]] = false;
                    $toDisable[$rp[$this->_helper->getEntityField()]] = $rp[$this->_helper->getEntityField()];
                }
            }*/

            $check = [];
            if (intval($store["store_id"]) == 0 && $imageSimpleCheck) {
                $sqlImage = "SELECT a.".$this->_helper->getEntityField().", a.sku FROM `catalog_product_entity` AS a 
                             LEFT JOIN `catalog_product_entity_media_gallery_value_to_entity` AS b 
                             ON a.".$this->_helper->getEntityField()." = b.".$this->_helper->getEntityField()."
                             WHERE b.value_id IS NULL
                             AND a.".$this->_helper->getEntityField()." IN (".implode(",",$simpleProducts).")";

                $resultsImages = $connection->fetchAll($sqlImage);
                foreach ($resultsImages as $ri){
                    $problems[$ri['sku']]['images'][$store["code"]] = false;
                    $toDisable[$ri[$this->_helper->getEntityField()]] = $ri[$this->_helper->getEntityField()];
                }
            }

            $check = [];
            if (intval($store["store_id"]) == 0 && $imageCheck) {
                $sqlImage = "SELECT a.".$this->_helper->getEntityField().", a.sku FROM `catalog_product_entity` AS a 
                             LEFT JOIN `catalog_product_entity_media_gallery_value_to_entity` AS b 
                             ON a.".$this->_helper->getEntityField()." = b.".$this->_helper->getEntityField()."
                             WHERE b.value_id IS NULL
                             AND a.type_id = 'configurable'
                             AND a.".$this->_helper->getEntityField()." IN (".implode(",",$configurableProducts).")";

                $resultsImages = $connection->fetchAll($sqlImage);
                foreach ($resultsImages as $ri){
                    $problems[$ri['sku']]['images'][$store["code"]] = false;
                    $toDisable[$ri[$this->_helper->getEntityField()]] = $ri[$this->_helper->getEntityField()];
                }
            }

            $check = [];
            if (intval($store["store_id"]) == 0 && $categorySimpleCheck) {
                $sqlCategory = "SELECT cpe.".$this->_helper->getEntityField().", cpe.sku
                                FROM catalog_product_entity as cpe
                                LEFT JOIN catalog_category_product as ccp
                                on cpe.entity_id = ccp.product_id
                                WHERE category_id IS NULL
                                AND cpe.".$this->_helper->getEntityField()." IN (".implode(",",$simpleProducts).")";

                $resultsCategories = $connection->fetchAll($sqlCategory);
                foreach ($resultsCategories as $rc){
                    $problems[$rc['sku']]['category'][$store["code"]] = false;
                    $toDisable[$rc[$this->_helper->getEntityField()]] = $rc[$this->_helper->getEntityField()];
                }
            }

            $check = [];
            if (intval($store["store_id"]) == 0 && $categoryConfCheck) {
                $sqlCategory = "SELECT cpe.".$this->_helper->getEntityField().", cpe.sku
                                FROM catalog_product_entity as cpe
                                LEFT JOIN catalog_category_product as ccp
                                on cpe.entity_id = ccp.product_id
                                WHERE category_id IS NULL
                                AND cpe.".$this->_helper->getEntityField()." IN (".implode(",",$configurableProducts).")";

                $resultsCategories = $connection->fetchAll($sqlCategory);
                foreach ($resultsCategories as $rc){
                    $problems[$rc['sku']]['category'][$store["code"]] = false;
                    $toDisable[$rc[$this->_helper->getEntityField()]] = $rc[$this->_helper->getEntityField()];
                }
            }
        }

        if(count($toDisable)) {
            $disableSql = "update catalog_product_entity_int set value= 2 
                           WHERE attribute_id = (
                                 SELECT eav.attribute_id FROM `eav_attribute` AS eav
                                 WHERE eav.`attribute_code` LIKE 'status' AND eav.entity_type_id = 4
                           )  
                           And ".$this->_helper->getEntityField()." in (" . implode(",", $toDisable) . ")";
            $connection->query($disableSql);
            $disableSql = "update catalog_product_entity_int set value=1 
                           WHERE attribute_id = (
                                 SELECT eav.attribute_id FROM `eav_attribute` AS eav
                                 WHERE eav.`attribute_code` LIKE 'to_check' AND eav.entity_type_id = 4
                           )  
                           And ".$this->_helper->getEntityField()." in (" . implode(",", $toDisable) . ")";
            $connection->query($disableSql);
        }

//        foreach ($products as $product){
//            if(isset($product['manual_status']) && intval($product['manual_status'])){
//                continue;
//            }
//            $check = [];
//            $type = $product['type_id'];
//            foreach ($stores as $store) {
//                $product = $this->_productRepository->get($product['sku'], true, $store["store_id"], true);
//
//
//                if($priceCheck && $type != 'configurable'){
//                    if(!isset($product['price']) || (isset($product['price']) && floatval($product['price']) < 1)){
//                        $check['price'][$store["code"]] = false;
//                    }
//                }
//                /*elseif($priceCheck && $type == 'configurable' && intval($store["store_id"]) == 0){
//                    $res = $this->checkConfigurablePrice($product);
//                    if(!$res){
//                        $check['price_configurable'][$store["code"]] = false;
//                    }
//                }*/
//
//                if(intval($store["store_id"]) == 0) {
//                    $categoryCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_categories', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
//
//                    if ($categoryCheck) {
//                        if (!isset($product['category_ids']) || (isset($product['category_ids']) && count($product['category_ids']) <= 0)) {
//                            $check['category'][$store["code"]] = false;
//                        }
//                    }
//
//                    $imageCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_image', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
//
//                    if ($imageCheck) {
//                        if(!$product->getImage() || trim($product->getImage()) == 'no_selection'){
//                            $check['images'][$store["code"]] = false;
//                        }
//                        /*if (!isset($product['media_gallery']) || (isset($product['media_gallery']) && count($product['media_gallery']) <= 0)) {
//                            if (!isset($product['media_gallery']['images']) || (isset($product['media_gallery']['images']) && count($product['media_gallery']['images']) <= 0)) {
//                                $check['images'][$store["code"]] = false;
//                            }
//                        }*/
//                    }
//                }
//            }
//            if(count($check) > 0) {
//                $attributesData['status'] = 0;
//                $attributesData['to_check'] = 1;
//                if(!isset($product['name']) || (isset($product['name']) && trim($product['name']) == '')){
//                    $attributesData['name'] = 'NO NAME';
//                }
//                try {
//                    $this->_objectManager->get('Magento\Catalog\Model\Product\Action')->updateAttributes([$product['entity_id']], $attributesData, 0);
//                    //$this->_productRepository->save($product);
//                }
//                catch (\Exception $e){
//                    $this->_logger->info('Product: '.$product['sku'] . " Error: " . $e->getMessage());
//                }
//                $problems[$product['sku']] = $check;
//            }
//        }

        $this->sendReport($problems);
        $this->_logger->info('End Price Check');
    }

    public function sendReport($problems){
        /* Here we prepare data for our email  */
        if(!isset($sender['name']) || !isset($sender['email']))
        {
            $sender = "general";
        }
        $sender = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_email/checkproducts_email_sender', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $senderEmail = $this->_scopeConfig->getValue('trans_email/ident_' . $sender . '/email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $senderName = $this->_scopeConfig->getValue('trans_email/ident_' . $sender . '/name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $receiversEmail = explode(',', $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_email/checkproducts_email_receiver', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));


        /* Receiver Detail  */
//        $receiverInfo = [
//            'name' => '',
//            'email' => 'luigi+test@filoblu.com'
//        ];


        /* Sender Detail  */
        $senderInfo = [
            'name' => $senderName,
            'email' => $senderEmail,
        ];


        /* Assign values for your template variables  */
        $emailTemplateVariables = [];
        $emailTempVariables['cron'] = 'Price Check';

        $tableBodyPrices = '';
        $tableBodyCategory = '';
        $tableBodyImages = '';

        foreach($problems as $key => $values){

            foreach ($values as $value => $stores ){
                if($value == 'price') {
                    $tableBodyPrices .= '<tr>';
                    $tableBodyPrices .= '<td>'. $key .'</td>';
                    $tableBodyPrices .= '<td>Price less than 0</td>';
                    $tableBodyPrices .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                    $tableBodyPrices .= '</tr>';
                }
                elseif ($value == 'price_configurable'){
                    $tableBodyCategory .= '<tr>';
                    $tableBodyCategory .= '<td>'. $key .'</td>';
                    $tableBodyCategory .= '<td>All children have zero price</td>';
                    $tableBodyCategory .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                    $tableBodyCategory .= '</tr>';
                }
                elseif ($value == 'category'){
                    $tableBodyCategory .= '<tr>';
                    $tableBodyCategory .= '<td>'. $key .'</td>';
                    $tableBodyCategory .= '<td>Without category</td>';
                    $tableBodyCategory .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                    $tableBodyCategory .= '</tr>';
                }
                elseif ($value == 'images'){
                    $tableBodyImages .= '<tr>';
                    $tableBodyImages .= '<td>'. $key .'</td>';
                    $tableBodyImages .= '<td>Image not found</td>';
                    $tableBodyImages .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                    $tableBodyImages .= '</tr>';
                }
            }
        }


        $emailTempVariables['tBodyPrices'] = $tableBodyPrices;

        $emailTempVariables['tBodyCategory'] = $tableBodyCategory;
        $emailTempVariables['tBodyImages'] = $tableBodyImages;

        /* call send mail method from helper or where you define it*/
        $this->_objectManager->get('FiloBlu\Checkproducts\Helper\Email')->sendCustomEmail(
            $emailTempVariables,
            $senderInfo,
            $receiversEmail,
            'filoblu_checkproducts/checkproducts_email/checkproducts_email_template_price'
        );
    }

    public function checkConfigurablePrice($configurableProduct){
        $childProducts = $this->_linkManagement->getChildren($configurableProduct['sku']);
        foreach ($childProducts as $children){
            $childrenPrice = true;
            $stores = $this->_storeRepository->getList();
            foreach ($stores as $store) {
                $product = $this->_productRepository->get($children['sku'], true, $store["store_id"], true);
                if(!isset($product['price']) || (isset($product['price']) && floatval($product['price']) < 1)){
                    $childrenPrice = false;
                    Break;
                }
            }
            if($childrenPrice == true){
                return true;
            }
        }
        return false;
    }

}